// Module name: AVS
.kernel PL3_TO_PA
.code

#include "VP_Setup.g4a"
#include "Set_Layer_0.g4a"
#include "Set_AVS_Buf_0123_PL3.g4a"
#include "PL3_AVS_Buf_0.g4a"
#include "PL3_AVS_Buf_1.g4a"
#include "PL3_AVS_Buf_2.g4a"
#include "PL3_AVS_Buf_3.g4a"
#include "Save_AVS_PA.g4a"        
#include "EOT.g4a"

.end_code  

.end_kernel
