/*
 * Copyright © <2010>, Intel Corporation.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sub license, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
 * IN NO EVENT SHALL PRECISION INSIGHT AND/OR ITS SUPPLIERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */
// Modual name: ME_header.inc
//
// Global symbols define
//

/*
 * Constant
 */

define(`INTER_PART_MASK',               `0x7e000000')
define(`mpeg2_ref',		`r83')
define(`pic_ref',		`r82')
define(`INTRA16_DC_PRED',      `0xBB')

/* Cost center ref */
define(`mv_cc_ref',		`r81')

define(`tmp_vme_wb0',              `r100')
define(`tmp_vme_wb1',              `r101')
define(`tmp_vme_wb2',              `r102')
define(`tmp_vme_wb3',              `r103')
define(`tmp_vme_wb4',              `r104')
define(`tmp_vme_wb5',              `r105')
define(`tmp_vme_wb6',              `r106')
define(`tmp_vme_wb7',              `r107')
define(`tmp_vme_wb8',              `r108')
define(`tmp_vme_wb9',              `r109')
